<?php
namespace APS;

require_once "aps/2/apsc.php";
require_once "aps/2/types.php";
require_once "aps/2/schema.php";

//define('APS_PATH', dirname(__FILE__));
//function autoLoadTypes($className)
//{
//    $file = APS_PATH."/types/".strtolower($className).".php";
//    if (file_exists($file))
//    {
//        require_once($file);
//        TypeLibrary::registerClass($className);
//        print_r(TypeLibrary::getSchemaByTypeName($className));
//    }
//}
//spl_autoload_register('autoLoadTypes');

class Request
{
	private static $apsc;

	public static function getController($instanceId = NULL)
	{
		if($instanceId)
			$_SERVER['HTTP_APS_INSTANCE_ID'] = $instanceId;

		if (!self::$apsc)
			self::$apsc = new ControllerProxy(\APS\Proto::getControllerURI(), \APS\Proto::getSession());
			
		return self::$apsc;
	}

    public static function setController(ControllerProxy $controller)
    {
        self::$apsc = $controller;
    }
    
    public static function getPhase()
    {
    	$k = 'HTTP_APS_REQUEST_PHASE';
    	return key_exists($k, $_SERVER) ? $_SERVER[$k] : null;
    }
}

class ResourceMeta
{
	var $type;
	var $id;
	var $links;
	var $changed;
	var $status;	
	
	public function __construct($type, $id = NULL)
	{
		$this->type = $type;
		$this->id = $id;
	}
	
	public function log($severity, $message, array $parameters = NULL, $time = NULL) {
		return \APS\Request::getController()->logById($this->id, $severity, $message, $parameters, $time);
	}
	
	public function getLog() {
		return \APS\Request::getController()->getLogById($this->id);
	}
	
	public function deleteLog() {
		return \APS\Request::getController()->deleteLogById($this->id);
	}
}

class Link
{
	var $id;
	var $href;
	var $link;
	var $name;
	var $collection;
	
	public function isCollection() 
	{
		return $this->collection;
	}

	public function fromXML($node)
	{
		if ($node->nodeType != \XML_ELEMENT_NODE)
			throw new \Exception("Wrong node to parse Link");
			
		$this->name = $node->tagName;
		$this->link = $node->getAttributeNodeNS( APS2_NS_URI, 'link' )->value;
		$this->href = $node->getAttributeNodeNS( APS2_NS_URI, 'href' )->value;
		$this->collection = (bool)$node->getAttributeNodeNS( APS2_NS_URI, 'collection' );
		$this->id = $this->isCollection() ? null : $this->getIdByHref($this->href);
	}
	
	public function fromJSON($k, $tree)
	{
		if (!isset($tree->aps) || !isset($tree->aps->link))
			throw new \Exception("Wrong tree to parse Link");
		
		$this->name = $k;
		$this->link = $tree->aps->link;
		$this->href = $tree->aps->href;
		$this->collection = $tree->aps->link == "collection";
		$this->id = $this->isCollection() ? null : $this->getIdByHref($this->href);
	}
	
	public function __construct($obj = NULL, $name = NULL, $base = NULL)
	{
		if (!$obj)
			return;
			
		if ($obj instanceof ResourceBase || $obj instanceof ResourceProxy)
			$this->fromResource($obj, $name, $base);
		elseif ($obj instanceof \stdClass)
			$this->fromJSON($name, $obj);
		elseif (preg_match('/[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}/i', $obj))
			$this->id = $obj;
		else
			$this->fromXML($obj);
	}
	
	public function fromResource($res, $name, $base)
	{
		$schema = TypeLibrary::getSchemaByTypeId($base->aps->type);
		if (!isset($schema->relations[$name]))
			throw new \Exception("Unknown relation '$name' for resource '".$res->aps->id."'");
			
		$rel = $schema->relations[$name];
		
		$this->name = $name;
		if (!$rel->collection)
		{
			$this->id = $res->aps->id;
			$this->link = $rel->required ? "strong" : "weak";
		}
		else
		{
			$this->collection = true;
			$this->link = "collection";
		}
		$this->href = "/" . Proto::resourcePath($res->aps->id);
	}
	
	static function loadLinksXML($doc)
	{
		$root = $doc->documentElement;
		$ret = array();
		foreach($root->childNodes as $n) 
		{
			if ($n->nodeType == \XML_ELEMENT_NODE)
				if ($n->getAttributeNodeNS ( APS2_NS_URI, 'link' ))
				{
					$link = new Link();
					$link->fromXML($n);
					$ret[] = $link;
				}
		}
		return $ret;		
	}
	
	public function toXML($root)
	{
		$node = $root->ownerDocument->createElement($this->name);
		$node->setAttributeNS( APS2_NS_URI, 'aps:link', $this->link );
		$node->setAttributeNS( APS2_NS_URI, 'aps:href', $this->href );
		$root->appendChild($node);
	}
	
	public function toArray()
	{
		$ret = new \stdClass();
		$ret->link = $this->link;
		$ret->href = $this->href;
		
		if ($this->id)
			$ret->id = $this->id;
		
		if ($this->isCollection())
			$ret->link = 'collection';

		return array('aps' => $ret);
	}
	
	private function getIdByHref($href)
	{
		preg_match('/([\w-]+)$/', $href, $matches);
		if (!$matches)
				throw new \Exception("Can't parse resource ID from: '$href'");
			
		return $matches[0];		 
	}
	
	/**
	 * Create specific link by collection link template
	 * @param string $id - id of resource on other end of link
	 */
	public function linkTo($id)
	{
		if (!$this->isCollection())
			throw new \Exception("linkTo() called not on collection link");

		$new = new Link();
		$new->name = $this->name;
		$new->link = $this->link;	
		$new->id = $id;
		$new->href = "/resources/".$id;
		
		return $new;		
	}
}

class ResourceBase
{
	// APS resource meta $this->aps->id, $this->aps->type
    /** @var stdObject */
	var $aps;

	// @desc resource backup before 'configure' or 'provision'
	private $originalResource = null;

	public function id()
	{
		return $this->aps->id;
	}
	
	public function type()
	{
		return $this->aps->type;
	}
	
	protected function _copy($r)
	{
		foreach ($r as $k => $v)
			$this->$k = $v;
	}
	
	public function __construct() {
		$r = new \ReflectionClass($this);
		$comment = $r->getDocComment();
		preg_match("/\@type\(\"(.*?)\"\)/", $comment, $type);
		if(isset($type[1])) {
			$this->aps = new \APS\ResourceMeta($type[1]);
			$className = get_class($this);
			$this->aps->elementName = $className;
			$this->aps->elementNS = NULL;		
			$this->aps->links = array();
		}
	}

	public function __call($method, $args)
	{
		$schema = TypeLibrary::getSchemaByTypeId($this->aps->type);
		foreach($schema->operations as $implemented_method_name => $method_info) {
			if ($implemented_method_name != $method) continue;
				$pathAndBody = $method_info->applyArguments($args);
				$path = $pathAndBody[0];
				$body = $pathAndBody[1];
				$result = Request::getController()->getIo()->sendRequest(
					$method_info->verb,
					Proto::resourcePath($this->aps->id, $path),
					$body
				);
				if (isset($method_info->response->type)) {
					// if type defined => it is json
					return json_decode($result);
				} else {
					return $result;
				}
		}
		throw new \Exception("unknown method [$method]");
	}
	
	function _prepare($method, &$args, $op)
	{
		$class = '\\'.get_class($this);
		$schema = TypeLibrary::getSchemaByTypeName($class);

		// do nothing for internal methods
		if (substr($method, 0, 1) == '_')
			return;

		// Find applicable operation
		$linkOrUnlink = false; 
		if (isset($schema->operations[$method]))
			$o = $schema->operations[$method];
		elseif(preg_match("/(.*)(Unlink|Link)$/", $method, $matches) && 
				array_key_exists($matches[1], $schema->relations)) {
			$linkType = $schema->relations[$matches[1]]->getType();
			$linkOrUnlink = true;
		}
		else			
			throw new \Rest\RestException(404, "Method {$class}::{$method} not found");

		// Override class method name for Async case
		if (Request::getPhase() == "async")
		{
			$op->override_method_name = $method."Async";
			if (!method_exists($this, $op->override_method_name))
				throw new \Rest\RestException(404, "Async Method {$class}::{$op->override_method_name} not defind");
		}
	
		$nargs = array();
		if ($linkOrUnlink) {
			$this->fillResource($args[TypeLibrary::PathIdName], false);
			if(isset($args['request_data'])) {
				$nargs[0] = $args['request_data'];
			} elseif(isset($args[TypeLibrary::LinkedResourceId])) {
				$this->checkGUID($args[TypeLibrary::LinkedResourceId]);
				if(TypeLibrary::isTypeRegistered($linkType)) {
					try {
                        $nargs[0] = Request::getController()->getResource($args[TypeLibrary::LinkedResourceId]);
					} catch (\Exception $e) {
						print "Failed to get resource '".$args[TypeLibrary::LinkedResourceId]."' from apsc: ".$e->getMessage()."\n";
					}
				} else {
					$nargs[0] = new ResourceProxy(new Link($args[TypeLibrary::LinkedResourceId]));//Request::getController()->getResource($args[TypeLibrary::LinkedResourceId]);
				} 
			}
		} else {
			if(!$o->static)
				$this->fillResource($args[TypeLibrary::PathIdName], true);
							
			foreach ($o->parameters as $k => $p)
			{
				if (!is_null($p->index) && isset($args[$p->name]))
					$nargs[$p->index] = $args[$p->name];
					
				if ($p->kind == "body" && isset($args['request_data']))
					$nargs[$p->index] = $args['request_data'];
			}
		}
		$args = $nargs;
	}

	function _prepareData($method, $data)
	{
		if (!in_array($method, array('provision', 'configure'))) {
			return;
		}

		if ('provision' == $method) {
			if (defined("APS_DEVELOPMENT_MODE")) {
				\APS\Logger::get()->debug("Request:\n$data");
			}
			\APS\TypeLibrary::fillResource($this, $data);
		}

		// save resource content, to compare and return resource only if any properties are changed by application
		$this->originalResource = $this->toString();
	}
	
	function fillResource($id, $newSession = false) {
		// Operating on object, load it from controller
		$this->checkGUID($id);
			
		try {
			$apsc = Request::getController();
			if ($newSession)
			{
				$apsc = clone $apsc;
				$apsc->resetSession();
			}
			$r = $apsc->getResource($id);
			$this->_copy($r);
		} catch (\Exception $e) {
			print "Failed to get resource '$id' from apsc: ".$e->getMessage()."\n";
		}
	}
	
	function checkGUID($id) {
		if (!preg_match('/[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}/i', $id))
			throw new \Rest\RestException(500, "Resource ID '$id' is not in GUID format");
	}
	
	function _postprocess($method, $args, $response)
	{
		if (substr($method, 0, 1) == '_')
			return $response;
		
		$class = get_class($this);
		$schema = TypeLibrary::getSchemaByTypeName($class);

		// Find applicable operation
		if ((!isset($schema->operations[$method])) && (!preg_match("/(.*)(Link|Unlink)$/", $method)))
			throw new \Rest\RestException(404, "Method $method not found for $class");
		
		switch($method)
		{
			case "retrieve":  // GET  /:id => return resource in body
				return $this;
			case "provision": // POST /    => return resource in body in case if it changed
			case "configure": // PUT  /:id => return resource in body in case if it changed
				if ($this->originalResource == $this->toString()) 
					return;
				else
					return $this;
		}
		
        if ('unprovision' == $method && $schema->isTopResource) {
            TypeLibrary::removeConfFile();
        }

        if ('unprovision' == $method || preg_match("/(.*)Unlink$/", $method)) {
            throw new \Rest\NoContent();
        }
		return $response;
	}

	function _getSchema()
	{
		$class = get_class($this);
		$schema = TypeLibrary::getSchemaByTypeName($class);
		return isset($schema->base) ? $schema->base : $schema;
	}

	function _getDefault()
	{
		$ret = array();
		$props = 0;
		foreach ($this->toArray() as $k =>$v)
		{
			if ($v == null) // skip null values
				continue;
				
			$ret[$k] = $v;
			
			if ($k != "aps")
				$props++;
		}
		
		if (!$props) // send 404 in case of no defaults
			throw new \Rest\RestException(404, "No defaults");

		return $ret;
	}
	
	public function retrieve()
	{
	}

	public function provision()
	{
	}

	public function configure($resource)
	{
		$this->_copy($resource);
	}

	public function unprovision()
	{
	}
				
	public function __set($name, $value)
    {
        $this->$name = $value;
        if (preg_match('/^\w/', $name))
        	return;
 
        $this->aps->changed[$name] = true;
    }

	public function toXML()
	{
		$dom = new \DOMDocument();
		if (isset($this->aps->elementNS))
			$root = $dom->createElementNS( $this->aps->elementNS, $this->aps->elementName );
		else
			$root = $dom->createElement( $this->aps->elementName );
		$dom->appendChild($root);
		
		$root->setAttributeNS( APS2_NS_URI, 'aps:id',   $this->aps->id );
		$root->setAttributeNS( APS2_NS_URI, 'aps:type', $this->aps->type );
		
		serializeObjectXML($root, $this);
		
		if (isset($this->aps->links))
			foreach ($this->aps->links as $link)
				$link->toXML($root);

		return $dom;
	}
	
	public function toArray()
	{
		$schema = TypeLibrary::getSchemaByTypeName(get_class($this));

		$ret = array();
		
		if (isset($this->aps->type))
			$ret['aps'] = array('type' => $this->aps->type);
		else
			$ret['aps'] = array('type' => $schema->id);

		if (isset($this->aps->id))
			$ret['aps']['id'] = $this->aps->id;
		
		
		foreach($schema->getProperties() as $p) {
			if (property_exists($this, $p->name)) {
				$ret[$p->name] = $this->{$p->name}; // TODO: traverse down to skip null elements
			}
        	}
			
		if (isset($this->aps->links))
			foreach ($this->aps->links as $link)
				$ret[$link->name] = $link->toArray();
				
		return $ret;
	}

	/**
	 * Serialize to string
	 * @param string $representation - either JSON or XML (default)
	 */
	public function toString($representation = TypeLibrary::JSON)
	{
		if ($representation == TypeLibrary::JSON) 
		{
			return json_format($this->toArray());
		} 
		elseif ($representation == TypeLibrary::XML)
		{
			$dom = $this->toXML();
			$dom->formatOutput = true;
			return $dom->saveXML();
		} else
			throw new \Exception("Unknown representation");
	}		

	public function __toString()
	{
		return $this->toString();
	}
	
	public function isChanged()
	{
		return (bool)$this->aps->changed;
	}


	/**
	 * Evaluiates php code and returns the result
	 * Example: evalCode('$this->db->prefix') returns the value of $this->db->prefix.
	 * @param string $representation - either JSON or XML (default)
	 */
	public function evalCode($var, $error_comment = "") {
		$result = "";
		eval('try { $result = $this->'.$var.'; } catch (\Exception $e) { throw new \Exception ("variable $var was not set '.$error_comment.'"); }');
		return $result;
	}

}

class ResourceProxy
{
	var $link;
	var $resource;
	
	public function __construct($link)
	{
		$this->link = $link;
	}

	public function __set($k, $v)
	{
		$this->load();
		$this->resource->$k = $v;
		if ($v instanceof ResourceBase)
			$this->aps->links[$k] = new Link($v, $k);
	}
	
	public function __get($k)
	{
		$this->load();
		return $this->resource->$k;
	}
	
	public function load()
	{
		if ($this->resource)
			return;
			
		$this->reload();
	}
	
	public function reload()
	{
		if ($this->link->id) {
			$this->resource = Request::getController()->getResource($this->link->id);
		} else if ($this->link->href) {
			$resources = Request::getController()->getResources("", $this->link->href);
			if (count($resources)) {
				$this->resource = $resources[0];
				$this->link->id = $resources[0]->aps->id;
			}
		}
	}
	
    public function __call($method, $args)
    {
    	$this->load();
    	
        if (method_exists($this->resource, $method)) {
            return call_user_func_array(
                array($this->resource, $method),
                $args
            );
        }
        // Looking for the method in implemented schemas
        $schema = TypeLibrary::getSchemaByTypeId($this->resource->aps->type);

        foreach($schema->implements as $implemented_type) {
            $implemented_schema = TypeLibrary::getSchemaByTypeId($implemented_type);
            if (!$implemented_schema) continue;
            foreach($implemented_schema->operations as $implemented_method_name => $method_info) {
                if ($implemented_method_name != $method) continue;

                // Method found. Call and return its result
                $pathAndBody = $method_info->applyArguments($args);
                $path = $pathAndBody[0];
                $body = $pathAndBody[1];

                $result = Request::getController()->getIo()->sendRequest(
                        $method_info->verb,
                        Proto::resourcePath($this->resource->aps->id, $path), 
                        $body
                    );
                if (isset($method_info->response->type)) {
                    // type defined => it is json
                    return json_decode($result);
                } else {
                    return $result;
                }

            }
        }

        throw new \Exception("unknown method [$method]");
    }
    
    public function __toString()
    {
    	$this->load();
    	return $this->resource->toString();
    }
}

class CollectionProxy implements \ArrayAccess, \Iterator, \Countable
{
	// @desc base resource ID
	private $baseId;
	
	// @desc collection members, loaded on demand
    private $list;
    
    // @desc postion for iterator
    private $position;
    
    // @desc link object
    private $link;

    // @desc rql filter to apply to original set
    public $rqlFilter = null;
    
    /**
     * Constructor for CollectionProxy
     * @param string $baseId - base resource ID
     * @param Link $link - link structure
     */
    public function __construct($baseId, $link) 
    {
    	$this->baseId = $baseId;
        $this->link = $link;
    }

    // load collection contents if required
    private function checkLoad()
    {
    	if ($this->list == null)
    	{
    		$resources = Request::getController()->getResources($this->rqlFilter, $this->link->href);
    		
    		$this->list = array();
    		foreach ($resources as $r)
			{
				$link = $this->link->linkTo($r->id());
				$this->list[] = new ResourceProxy($link);
			} 		
    	}
    }
    
    // force re-read collection contents
    public function reload()
    {
    	$this->list = null;
    	$this->checkLoad();
    }
    
    // update of links is disabled
    private function checkUpdate()
    {
		throw new \Exception("Use link() to modify collections");
    }
    
    // ArrayAccess
    public function offsetGet($offset) 
    {
		$this->checkLoad();
    	return isset($this->list[$offset]) ? $this->list[$offset] : null;
    }

    // ArrayAccess
    public function offsetExists($offset) 
    {
    	$this->checkLoad();
        return isset($this->list[$offset]);
    }
    
    // ArrayAccess
    public function offsetSet($offset, $value) 
    {
    	$this->checkUpdate();
    }

    // ArrayAccess
    public function offsetUnset($offset) 
    {
    	$this->checkUpdate();
	}

	// Iterator
 	function rewind() {
        $this->position = 0;
    }

    // Iterator
    function current() {
        return $this[$this->position];
    }

    // Iterator
    function key() {
        return $this->position;
    }

    // Iterator
    function next() {
        ++$this->position;
    }

    // Iterator
    function valid() {
    	$this->checkLoad();
        return isset($this->list[$this->position]);
    }
    
    // Countable
    function count()
    {
	    $this->checkLoad();
    	return count($this->list);
    }
    
    function filter($filter)
    {
    	if (!$filter)
	    	return $this;
	    	
    	$ret = clone $this;
    	$ret->rqlFilter = ($ret->rqlFilter != null) ? "and(".$ret->rqlFilter.",$filter)" : $filter;
    	$ret->reload();
    	return $ret;
    }
}


class EventSubscription
{
	const Linked 	= "http://aps-standard.org/core/events/linked";
	const Unlinked	= "http://aps-standard.org/core/events/unlinked";
	const Changed	= "http://aps-standard.org/core/events/changed";
	const Removed	= "http://aps-standard.org/core/events/removed";
	const Available	= "http://aps-standard.org/core/events/available";

	/** @description("Event subscription identifier") */
    public $id;

	/** @description("Event type identifier") */
    public $event;

    /** @description("Either resource or type") */
    public $source; 
    
    /** @description("Relation id for events link/unlink events") */
    public $relation;
         
    /** @description("Operation name to handle Event, should accep only parameter of type EventNotification") */
	public $handler;
	
	public function __construct($event = null, $handler = null)
	{
		$this->event = $event;
		$this->handler = $handler;
	}
	
	public function toJSON()
	{
		$req = clone $this;
		if ($this->source instanceof ResourceBase)
		{ 
			// Subscribe particular resource
			$req->source = new \stdClass();
			$req->source->id = $this->source->aps->id;
		} 
		elseif (gettype($this->source) == 'string')
		{ 
			// Subscribe by type
			$req->source = new \stdClass();
			$req->source->type = $this->source;
		}
		
		foreach (get_object_vars($this) as $k => $v)
			if (is_null($req->$k))
				unset($req->$k);
		
		return json_encode($req);
	}
	
	public function fromJSON($json)
	{
		if (gettype($json) == 'string')
			$rsp = json_decode($json);
		
		foreach (get_object_vars($this) as $k => $v)
			if (isset($rsp->$k)) $this->$k= $v;
			
		return $this;
	}
}
