<?php
namespace Rest;

class JsonFormat implements iFormat
{
    const MIME ='application/json';
    const EXTENSION = 'json';
    
    public function getMIMEMap()
    {
        return array(self::EXTENSION=>self::MIME);
    }
    
    public function getMIME()
    {
        return  self::MIME;
    }
    
    public function encode($data, $human_readable = FALSE)
    {	
    	if (is_object($data) && method_exists($data, 'toArray'))
    		$ret = $data->toArray();
    	else
    		$ret = $data;
    	
        return $human_readable ? json_format($ret) : json_encode($ret);
    }
    
    public function decode($data, $type = NULL) {
    	$obj = json_decode($data);
        if (function_exists( 'json_last_error' ) && (json_last_error() != \JSON_ERROR_NONE)) 
        {
            $message = null;
            switch (json_last_error()) {
                case \JSON_ERROR_DEPTH :
                    $message = 'maximum stack depth exceeded';
                    break;
                case \JSON_ERROR_STATE_MISMATCH :
                    $message = 'underflow or the modes mismatch';
                    break;
                case \JSON_ERROR_CTRL_CHAR :
                    $message = 'unexpected control character found';
                    break;
                case \JSON_ERROR_SYNTAX :
                    $message = 'malformed JSON';
                    break;
                case \JSON_ERROR_UTF8 :
                    $message = 'malformed UTF-8 characters, possibly incorrectly encoded';
                    break;
                default :
                    $message = 'unknown error';
                    break;
            }
            throw new RestException ( 400, 'Error parsing JSON, ' . $message );
        } 
        else if ( strlen($data) && $obj === NULL || $obj === $data)
        {
            throw new RestException ( 400, 'Error parsing JSON' );
        }

        try
        {        	
        	if (is_object($obj) && isset($obj->aps) && isset($obj->aps->type))
			{	// APS resource decoding using resource factory
				$resource = \APS\TypeLibrary::newResourceByTypeId($obj->aps->type);
				$obj = \APS\TypeLibrary::parseResourceJSON($resource, $obj);	
			}
			
			if(is_object($obj) && @class_exists($type)) {
				$schema = \APS\TypeLibrary::getSchemaByTypeName($type);
				$object = \APS\unserializeObjectFromJSON($obj, $schema, new $schema->name);
				return $object;
			}
			
			// TODO: other structures by type factory
        } 
        catch (\Exception $ex)
        {
        	 throw new RestException ( 400, 'Failed to parse aps object: '.$ex->getMessage() );
        }
        
        return $obj;
    }
}
?>
