<?php
namespace Rest;

class XmlFormat implements iFormat
{
    const MIME ='application/xml';
    const EXTENSION = 'xml';

    public function getMIMEMap()
    {
        return array(self::EXTENSION => self::MIME);
    }
    
    public function getMIME()
    {
        return self::MIME;
    }
    
	static private function isVector($var) 
	{ 
		return count(array_diff_key($var, range(0, count($var) - 1))) == 0; 
	}

    public function encode($data, $human_readable=FALSE)
    {
        if (is_object($data) && method_exists($data, 'toXML'))
        {
        	$doc = $data->toXML();
        } 
        else
        {
        	$doc = new \DOMDocument();
        	$root = $doc->createElement( 'data' ); // TODO: get name of root element
        	$doc->appendChild($root);
        	
        	$obj = $data;
        	if (is_array($data) && !self::isVector($data))
        	{
        		$obj = new \stdClass();
        		foreach ($data as $k =>$v)
        			$obj->$k = $v;	
        	}
        	
        	serializeObjectXML($root, $obj);
        }
        $doc->formatOutput = $human_readable;
        return $doc->saveXML();
    }
    
    public function decode($data) {
    	try {
	    	$dom = new \DOMDocument();
			$dom->loadXML($data);
	
			$root = $dom->documentElement;
		
			if ($root->hasAttributeNS(APS2_NS_URI, 'type')) // has @aps:type
				return \APS\TypeLibrary::parseResource($data);
		
			$ret = new \stdClass(); // TODO: typed value here
		
			return unserializeObject($root, $ret); // TODO: schema reference here
        } 
        catch (\Exception $ex)
        {
            throw new RestException ( 400, 'Error parsing XML: '.$ex->getMessage() );
        }
    }
}

?>