<?php
namespace APS;

define('APS_CHECK_CONTROLLER_CERTIFICATE', true);
define('MIN_SUPPORTED_PHP_VERSION', '5.3.3');
define('APS_PHP_RUNTIME_ROOT', substr(__FILE__, 0, strrpos(__FILE__, 'aps'.DIRECTORY_SEPARATOR.'2'.DIRECTORY_SEPARATOR.'runtime.php')));


/**
 * Runtime entry point
 * This file to be included as require_once('aps/2/runtime.php');
 * After that correct end-point will be exposed from appropriate file
 * 
 */

require_once 'aps/2/aps.php';
require_once 'aps/2/rest.php';
require_once 'aps/2/formats/xmlFormat.php';
require_once 'aps/2/formats/jsonFormat.php';
require_once 'aps/2/formats/simpleFormat.php';
require_once 'aps/2/utils.php';

function apsErrorHandler($errno, $errstr, $errfile, $errline)
{
    if (!(error_reporting() & $errno)) {
        // This error code is not included in error_reporting
        return;
    }
      
    error_log("[$errno] $errstr at $errfile:$errline");        
    throw new \Exception("$errstr at $errfile:$errline");
    
    return false;
}

if(!defined('PHALANGER'))
	set_error_handler('\APS\apsErrorHandler');

class Loader {
	private static $home;
	private $entryFile;
	
	public static function getHome() {
		return Loader::$home;
	}
	
	public function init() {
		$trace = debug_backtrace();
		$this->entryFile = $trace[count($trace) - 1]["file"];
		Loader::$home = dirname($this->entryFile);
	}
	
	//TODO: improve obtaining classes from file(with their corresponding namespaces)
	private function getPhpClasses() {
		$php_code = file_get_contents($this->entryFile);

		$tokens = token_get_all($php_code);

		$classes = array();
		$count = count($tokens);
		$namespace = '';
		for ($i = 2; $i < $count; $i++) {
			if($tokens[$i - 2][0] == T_NAMESPACE) {
				$namespace = $tokens[$i][1];
				while($tokens[$i + 1] != ';') {
					$namespace .= $tokens[$i + 1][1];
					$i++;
				}
			}
				
			if(    $tokens[$i - 2][0] == \T_CLASS 
				&& $tokens[$i - 1][0] == \T_WHITESPACE 
				&& $tokens[$i    ][0] == \T_STRING)
				$classes[] = '\\'.($namespace ? ($namespace.'\\') : '').$tokens[$i][1];
		}
		return $classes;
	}

	private function registerClasses($isExtendedMode = true) {
		$schema = null;
		$classes = $this->getPhpClasses();
		foreach($classes as $class) {
			$s = TypeLibrary::registerClass($class, $isExtendedMode);
			if ($s)
				$schema = $s;
		}
		
		return $schema;
	}
	
	private function generateClass($src)
	{
		$schemaSrc = file_get_contents($src);
		$schema = new Schema(array('schema' => $schemaSrc));	
		return $schema->toSource();
	}
	
	private function generateSchema()
	{
		chdir(Loader::getHome());
		return $this->registerClasses(false);
	}
	
	private function generatePackage($packageBase)
	{
		$dom = new \DOMDocument();
		$dom->loadXML(file_get_contents($packageBase . '/APP-META.xml'));
		$root = $dom->documentElement;
		$services = $dom->getElementsByTagName('service');
 		foreach ($services as $svc) 
 		{
 			$id = $svc->getAttribute ('id');
 			$schema = $svc->getElementsByTagName('schema');
 			if (count($svc) != 1)
 			{
 				error_log("Can't found schema for service id=\"$id\"");
 				return;
 			}
 			$path = $schema->item(0)->getAttribute ('path');
 			$src = $packageBase . '/' . $path;
 			$dst = "$id.php";
 			print "Generating service $id by $src to $dst\n";
 			$source = $this->generateClass($src);
 			file_put_contents($dst, $source);
 		}
	}
	
	private function handleHttpRequest()
	{
		if(php_sapi_name() === 'cli') 
		{
			error_log("Please, run php-runtime under webserver");
			return;
		}
		chdir(Loader::getHome());
		if(isset($_SERVER['HTTPS']) && defined('APS_CHECK_CONTROLLER_CERTIFICATE') && file_exists(TypeLibrary::getConfFile())) {
			# add 'SSLVerifyClient optional_no_ca' in apache config
			$savedCert = openssl_x509_parse(json_decode(file_get_contents(TypeLibrary::getConfFile()))->controllerCertificate);
			if (isset($_SERVER['SSL_CLIENT_CERT'])) {
				#  +ExportCertData option in apache config
				$currCert = openssl_x509_parse($_SERVER['SSL_CLIENT_CERT']);
				if ($savedCert != $currCert) {
					throw new \Exception("Unauthorized access. Saved controller certificate and received are different");
				}
			} else {
				$certProps = array('C', 'ST', 'L', 'O', 'OU', 'CN', 'T', 'I', 'S', 'D', 'UID', 'Email');
				foreach ($certProps as $prop) {
					if(self::is_prop_equal($savedCert['subject'], $_SERVER, $prop, 'SSL_CLIENT_S_DN_'.$prop) &&
						self::is_prop_equal($savedCert['issuer'], $_SERVER, $prop, 'SSL_CLIENT_I_DN_'.$prop)) {
						continue;
					}
					throw new \Exception("Unauthorized access. Saved controller certificate and received are different");
				}
			}
		}
		$s = $this->registerClasses(true);
		
		$r = new \Rest\RestService(TypeLibrary::makeRestRoutes());
		try {
			$r->setSupportedFormats('\Rest\SimpleFormat', '\Rest\JsonFormat');
			$r->handle();
		} catch (\Exception $exception) {
			$statusCode = $exception->getCode() ? $exception->getCode() : 500;
			$errorMessage = $exception->getMessage();
			$errorInfo = array(
				'code' => $statusCode,
				'type' => 'RuntimeException',
				'message' => \Rest\Response::$codes[$statusCode] . (!$errorMessage ? '' : ': ' . $errorMessage)
			);

			$r->sendError($errorInfo);
		}
	}

	public static function is_prop_equal($array1, $array2, $prop1, $prop2)
	{
		$prop1_exists = array_key_exists($prop1, $array1);
		if ($prop1 == 'Email' && !$prop1_exists) {
			// sometimes it's called 'emailAddress'
			$prop1 = 'emailAddress';
			$prop1_exists = array_key_exists($prop1, $array1);
		}
		$prop2_exists = array_key_exists($prop2, $array2);
		if (!$prop1_exists && !$prop2_exists) {
			return true;
		}
		if ($prop1_exists && $prop2_exists && $array1[$prop1] == $array2[$prop2]) {
			return true;
		}
		return false;
	}

	public function process()
	{
		if (isset($_SERVER['argv']) && isset($_SERVER['argv'][1]))
		{
			$args = array_slice($_SERVER['argv'], 1);
			switch (array_shift($args))
			{
				case '$class':
						print $this->generateClass(array_shift($args));
						return;
						
				case '$schema':
						print $this->generateSchema();
						return;
						
				case '$package':
						$this->generatePackage(array_shift($args));
						return;
			}
		}

		$this->handleHttpRequest();
	}

	public function __construct() {
		$this->init();
	}
	
	public function __destruct()
	{
		if(!defined('APS_RUNTIME_NO_PROCESS')) {
			try {
				$this->process();
			} catch (\Exception $e) {
				if (ob_get_level() > 0)
				{
					$fh = fopen("php://stderr", "a+");
					fwrite($fh, ob_get_contents());
					fclose($fh);
					ob_end_clean();
				}
		
				$code = (isset($e->code) && ($e->code > 0)) ? $e->code : 500;
				$human = \Rest\RestService::formatForHuman();
		
				if (key_exists('SERVER_PROTOCOL', $_SERVER))
				{
					header("{$_SERVER['SERVER_PROTOCOL']} $code Internal Error");
					header('Content-type: '.( $human ? "text/html" : "application/json" ));
				}
		
				$ret = array('code' =>  $code, 
						     'type' => get_class($e), 
						     'message' => $e->getMessage());
				
				Logger::get()->error(
						"Exception: code=$code, type=".get_class($e).", ".
						"message=".$e->getMessage()."\n".
						$e->getTraceAsString());
				
				if (defined("APS_DEVELOPMENT_MODE"))
					$ret['stacktrace'] = explode("\n", $e->getTraceAsString());

				$fmt = new \Rest\JsonFormat();
				print $fmt->encode($ret, $human);
			}
		}
	}
}

if(version_compare(PHP_VERSION, MIN_SUPPORTED_PHP_VERSION, '<'))
	exit("Need version greater or equal than ".MIN_SUPPORTED_PHP_VERSION);
// at end main file processing $loader will be destructed - do work here
$__loader = new Loader();

?>
