<?php
namespace org\standard\aps\types\sso\identity\management\provider;
class Token {
/**
 * @type(\org\standard\aps\types\sso\identity\management\provider\Identity)
 */
public $identity;
/**
 * @type(string)
 * @description("ID of identity-management resource")
 */
public $identityManagementResource;
/**
 * @type(string)
 */
public $scope;
/**
 * @type(string)
 * @format("date-time")
 * @description("last time when token was updated")
 */
public $updated;
/**
 * @type(string)
 * @format("date-time")
 * @description("expiration time of token")
 */
public $expiry;
/**
 * @type(\org\standard\aps\types\sso\identity\management\provider\object)
 */
public $providerInfo;
/**
 * @type(string)
 */
public $signature;

}
/**
 * Class IdentityProvider
 * @id("http://aps-standard.org/types/sso/identity-management/provider/1.0")
 * @implements("http://aps-standard.org/types/core/resource/1.0")
 * @structure(Token)
 */
class IdentityProvider extends \APS\ResourceBase
{
/**
 * @type(string)
 * @description("URI to operate over SSO protocol")
 */
public $URI;
/**
 * @type(string)
 * @pattern("SAML2|OpenID-Connect|LDAP3|.*")
 * @description("SSO protocol")
 */
public $protocol;
/**
 * @type(string[])
 * @description("List of security assertions satisfied by provider")
 */
public $complianceAssertions;
/**
 * @type(string)
 * @description("Public key to validate token sinature")
 */
public $publicSignatureKey;
/**
 * @link("http://aps-standard.org/types/sso/identity-management/account/1.0[]")
 */
public $accounts;

/**
 * @verb(GET)
 * @path("/challenge")
 * @return(object)
 * @param(string, query)
 * @param(string, query)
 * @param(string, query)
 */
public function getChallenge($scope, $identity, $requestingResource) {
	return \APS\Request::getController()->getIo()->sendRequest("GET", \APS\Proto::resourcePath($this->aps->id, "/challenge"));
}

/**
 * @verb(POST)
 * @path("/authenticate")
 * @return(Token)
 * @param(object, body)
 */
public function authenticate($challengeResponse) {
	return \APS\Request::getController()->getIo()->sendRequest("POST", \APS\Proto::resourcePath($this->aps->id, "/authenticate"), $challengeResponse);
}

/**
 * @verb(PUT)
 * @path("/tokens")
 * @return(Token)
 * @param(string, body)
 * @param(string, query)
 */
public function validateToken($encryptedToken, $requestingResource) {
	return \APS\Request::getController()->getIo()->sendRequest("PUT", \APS\Proto::resourcePath($this->aps->id, "/tokens"), $encryptedToken);
}

/**
 * @verb(DELETE)
 * @path("/tokens")
 * @param(string, body)
 */
public function deleteToken($encryptedToken) {
	return \APS\Request::getController()->getIo()->sendRequest("DELETE", \APS\Proto::resourcePath($this->aps->id, "/tokens"), $encryptedToken);
}
}
?>