<?php
namespace APS;

function generatePassword($length = 10, $spec_symbols = '!@#$%^&*()') {
	$digits = '0123456789';
	$lettersLow = 'abcdefghijklmnopqrstuvwxyz';
	$lettersUpper = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';

	$password = getRandChars(1, $digits);
	if (--$length > 0 && strlen($spec_symbols) > 0) $password .= getRandChars(1, $spec_symbols);
	if (--$length > 0) $password .= getRandChars(1, $lettersLow);
	if (--$length > 0) $password .= getRandChars(1, $lettersUpper);
	if (--$length > 0) $password .= getRandChars($length, $digits.$lettersUpper.$lettersLow.$spec_symbols);

	return str_shuffle($password);
}

function getRandChars($length, $chars = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ") {
    $randomBytes = openssl_random_pseudo_bytes($length, $strong);
    if (!$strong) {
        \APS\Logger::get()->warn("utils.php::generatePassword (openssl_random_pseudo_bytes): Non cryptographically strong algorith is used to generate password!");
    }
    $numchars = strlen($chars);
    $password = "";
    for ($i=0; $i<$length; ++$i) {
        $randomInt = hexdec(bin2hex($randomBytes[$i]));
        $password .= $chars[abs($randomInt % $numchars)];
    }
    return $password;
}

function printStderr($message)
{
	global $STDERR;
	if (! $STDERR) {
		$STDERR = fopen('php://stderr', 'w+');
	}
	fwrite($STDERR, $message);
}

function readTextFile($file)
{
	if (!file_exists($file)) {
		throw new \Exception("File ".$file." does not exist");
	}
	return fread(fopen($file, 'r'), filesize($file));
}

function writeFile($file, $content)
{
	$fp = fopen($file, 'wb');
	if (!$fp)
	{
		throw new \Exception("Unable to write file $file");
	}
	fputs($fp, $content, strlen($content));
	fclose($fp);
}

function phpQuote($val)
{
	$res_val = str_replace("\\", "\\\\", $val);
	$res_val = str_replace("'", "\\'", $res_val);
	return $res_val;
}

function modifyFile($file_source, $file_dest, $object)
{
	writeFile($file_dest, modifyContent($file_source, $object));
}

function modifyContent($file_source, $object)
{
	$file_content = readTextFile($file_source);
	return preg_replace("/@@[{]([^@{]+)[}]@@/e", "\$object->evalCode('\$1')", $file_content);
}

function removeDir($dir) {
	if(!file_exists($dir))
		return true;
	if(!is_dir($dir))
		return @unlink($dir);
	foreach(scandir($dir) as $item) {
		if($item == '.' || $item == '..')
			continue;
		if(!removeDir($dir.DIRECTORY_SEPARATOR.$item))
			return false;
	}
	return rmdir($dir);
}

$GLOBALS['reservedWords'] = array('Directory', 'stdClass', '__PHP_Incomplete_Class', 'Exception', 'ErrorException',
		'php_user_filter', 'Closure', 'Generator', 'self', 'static', 'parent', '__halt_compiler', 'abstract', 'and',
		'array', 'as', 'break', 'callable', 'case', 'catch', 'class', 'clone', 'const', 'continue', 'declare', 'default',
		'die', 'do', 'echo', 'else', 'elseif', 'empty', 'enddeclare', 'endfor', 'endforeach', 'endif', 'endswitch',
		'endwhile', 'eval', 'exit', 'extends', 'final', 'for', 'foreach', 'function', 'global', 'goto', 'if', 'implements',
		'include', 'include_once', 'instanceof', 'insteadof', 'interface', 'isset', 'list', 'namespace', 'new', 'or',
		'print', 'private', 'protected', 'public', 'require', 'require_once', 'return', 'static', 'switch', 'throw',
		'trait', 'try', 'unset', 'use', 'var', 'while', 'xor', '__CLASS__', '__DIR__', '__FILE__', '__FUNCTION__',
		'__LINE__', '__METHOD__', '__NAMESPACE__', '__TRAIT__');

function getTypeVersion($type) {
	$regex = '/\/(\d+\.\d+)$/';
	if (preg_match($regex, $type, $matches)) {
		return $matches[1];
	}
	return "1.0";
} 

function getNamespaceByType($type) {
	//Example : http://www.parallels.com/web/sampleresource/1.0 => \com\parallels\www\web\sampleresource\

	$regex = '/\:\/\/(.+?)\/(.+)\/\d+\.\d+$/';
	preg_match($regex, $type, $matches);
	$hostNameParts = explode('.', preg_replace('/[^\w]/', '.', $matches[1]));
	$pathParts = explode('.', preg_replace('/[^\w]/', '.', $matches[2]));
	for($i = 0; $i < count($hostNameParts); $i++) {
		if($hostNameParts[$i] == '-') {
			$hostNameParts[$i] = '_'; 
		} elseif (in_array($hostNameParts[$i], $GLOBALS['reservedWords'])) {
			$hostNameParts[$i] = '_'.$hostNameParts[$i];
		}
	}
	for($i = 0; $i < count($pathParts); $i++)  {
		if($pathParts[$i] == '-') {
			$pathParts[$i] = '_';
		} elseif (in_array($pathParts[$i], $GLOBALS['reservedWords'])) {
			$pathParts[$i] = '_'.$pathParts[$i];
		}
	}
	return '\\'.implode('\\', array_merge(array_reverse($hostNameParts), $pathParts));
}

Class FilesConfigurator {

	protected $application;
	protected $environment;

	function __construct($applicationObject, $environmentObject) {
		$this->application = $applicationObject;
		$this->environment = $environmentObject;
	}

	function writeConfigFile($template_file, $dest_path)
	{
		if (file_exists($dest_path)) {
			rename($dest_path, $dest_path.time());
			@unlink($dest_path.time());
		}
		modifyFile($template_file, $dest_path, $this->application);
	}

	function setWritePermissions($config_file)
	{
		if (file_exists($config_file)) {
			if(isLinux()) {
				chmod($config_file, 0777);
			} else {
				chmod($config_file, 666);
			}
		}
	}

	function setReadonlyPermissions($config_file)
	{
		if (file_exists($config_file)) {
			if(isLinux()) {
				chmod($config_file, 0444);
			} else {
				chmod($config_file, 444);
			}
		}
	}

	function isLinux() {
		return preg_match('/Windows/', $this->environment->Platform->os) ? 0 : 1;
	}
}


Class DatabaseConfigurator {

	protected $application;
	protected $db;
	protected $dbh;

	function __construct($applicationObject, $dbObject) {
		$this->application = $applicationObject;
		$this->db = $dbObject;

		if (!isset($this->db))
			throw new \Exception("Database relation is empty");
			
		if (!("" . $this->db->systemDSN))
			throw new \Exception("Database systemDSN is not defined");
	
		if (!("".$this->db->login))
			throw new \Exception("Database login is not defined");
	}

	private function connect()
	{
		$this->dbh = new \PDO($this->db->systemDSN, $this->db->login, $this->db->password);
		$this->dbh->setAttribute( \PDO::ATTR_ERRMODE, \PDO::ERRMODE_EXCEPTION );
	}
		
	function importSchema($schema)
	{
		$this->connect();
		$sql = modifyContent($schema, $this->application);
		$this->populateDB($sql);
		$this->dbh = null;
	}

	function importSchemas($schemas)
	{
		$this->connect();
		foreach($schemas as $schema) {
			$sql = modifyContent($schema, $this->application);
			$this->populateDB($sql);
		}
		$this->dbh = null;
	}

	private function execSQL($sql)
	{
		$this->dbh->exec($sql);
	}

	private function populateDB($sql)
	{
		$in_string = false;
		$in_comment = false;
		$start = 0;
		$len = strlen($sql);
		$backslash = false;

		for($i = 0; $i < $len; $i++)
		{
		if( ($sql[$i] == "#" || ($sql[$i] == "-" && $sql[$i+1] == "-")) && !$in_string)
			{
				$in_comment = true;
				continue;
			}

			if($sql[$i] == "\n" && $in_comment)
			{
				$in_comment = false;
				$start = $i+1;
				continue;
			}

			if($in_comment)
				continue;

			if($sql[$i] == ";" && !$in_string)
			{
				$statement = substr($sql, $start, $i - $start);
				if(!preg_match("/^[ \n\r\t]*$/", $statement)) {
					$this->execSQL($statement);
				}
				$start = $i+1;
			}

			if ($in_string) {
				if ( $sql[$i-1] == "\\" ) {
					$backslash = ! $backslash;
				} else {
					$backslash = false;
				}
			}
				
			if($sql[$i] == $in_string && ! $backslash)
			{
				$in_string = false;
			}
			elseif(($sql[$i] == '"' || $sql[$i] == "'") && !$in_string && ($i > 0 && $sql[$i-1] != "\\"))
			{
				$in_string = $sql[$i];
			}
		}

		if (!$in_comment)
		{
			$statement = substr($sql, $start);

			if(!preg_match("/^[ \n\r\t]*$/", $statement))
				$this->execSQL($statement);
		}
	}

}


?>
